;===================================================================
;
; PROGRAM:  mollosvd_spec
; 
; PURPOSE:  plots various spectral diagnostics
; INPUT  :  "observed" (reduced) channel maps
; OUTPUT :  spectra/plots
; USAGE  :  This is a multi-step process
;           (1) use mollosvd_make to generate "super-channel" maps,
;           i.e.high-resolution ppv cubes. These are the raw data and
;           are generally at higher spectral (and spatial) resolution
;           than observed data sets.
;           filename: mollosvd{axis}_{emitter}_{dump#}.fits
;           filename: mollosvd{axis}_{emitter}_{dump#}.head
;           (2) these ppv-cubes can be degraded to "observations",
;           using mollosvd_trans[late]. Degrading in velocity space
;           is done by binning, degrading in position space uses 
;           a Gaussian filter.
;           filename: mollosvd{axis}_{emitter}_{dump#}_c{channel#}s{beamwidth}e{noise}.fits
;           filename: mollosvd{axis}_{emitter}_{dump#}_c{channel#}s{beamwidth}e{noise}.head
;           The beamwidth is in pixels, and the noise level is in mJy.
; NOTES   : ilo:ihi and jlo:jhi area ALWAYS in the plane of sky.
;           Most of the RT part is from Rybicki & Lightman
;           Assumes Rayleigh-Jeans limit, i.e converts to brightness temperature
;--------------------------------------------------------------------

pro mollosvd_spec

  descr = 'c3f0g3b'

  if (descr eq 'Gf2') then begin
    legend0= '!6Gf2'
    models = 1
    mdl    = '3aaww00d0d1_g'
    dmp    = [19]
    dtdmp  = 8e-3
    convtime = 9.528e1  ;[Myr]
    idir   = 1
    lthick = [3]
    ltype  = [0]
    ptype  = [14,14,14,14,14,14,14]
    ctype  = [0,16,80,150,208,220,253]
    legend = ["!6Gf2, t = "+string(dtdmp*double(dmp)*convtime,format='(f4.1)')+' Myr']
    squeeze= 0.9
    nangle  = 1l
    whichmol= [0,3]
    sortindx= [0,1,2,3,4,5,6,7]
    beamwidth   = 2 ; in pixel
    nbins    = 40l
    noise   = 0.02 ; in Kelvin
    icen    = 70l
    jcen    = 110l
    iwdt    = 69l
    nbox    = 15l
    ;threshdata = 3.0
    invert=1
    contv    = 4
    colorvrms= 4
    colorvcen= 13
    color    = 1
    disk     = '~/proteus/run/collimflow/'
    odisk    = '~/proteus/run/collimflow/plots/'
    specdisk = '~/proteus/run/collimflow/tables/'
  endif

  if (descr eq 'c3f0g3b') then begin
    legend0= '!6c3f0g3b'
    models = 3
    mdl    = 'c3f0g3b_1'
    dmp    = [25,25,25]
    dtdmp  = 2.16e-3
    convtime = 9.528e2  ;[Myr]
    idir   = 2
    lthick = [3,3,3]
    ltype  = [0,0,0]
    ptype  = [14,14,14,14,14,14,14]
    ctype  = [0,16,80,150,208,220,253]
    legend = ["!6c3f0g3b, t = "+string(dtdmp*double(dmp)*convtime,format='(f4.1)')+' Myr']
    squeeze= 0.9
    angle  = [45,60,75]
    whichmol= [6]
    sortindx= [0,1,2,3,4,5,6,7]
    beamwidth   = 4 ; in pixel
    nbins    = 100l
    noise   = 0.02 ; in Kelvin
    icen    = 70l
    jcen    = 110l
    iwdt    = 69l
    subsize = 15l ; this is the size of the spectrum boxes
    ;threshdata = 3.0
    invert=1
    contv    = 4
    colorvrms= 4
    colorvcen= 13
    color    = 1
    disk     = '~/proteus/run/hvc/'
    odisk    = '~/proteus/run/hvc/plots/'
    specdisk = '~/proteus/run/hvc/tables/'
  endif
  ctable   = 39

;========================================================================

  convvelo = 9.118e-2 ;[km s^(-1)]
  convtime = 9.528e1  ;[Myr]
  convleng = 2.74e19  ;[cm]
  convpars = 8.896e0  ;[pc]
  convmass = 1.72e1   ;[Msol]
  convsurf = 2.74e19
  convkmsc = 1e5
  convintb = 9d20/(2d0*1.381d-16) ; for brightness emperature

;------------------------------------------------------------------------
; the molecule "database"
  nallmol     = 6
  nmol        = n_elements(whichmol)
;  speclabel = ['!6!e12!nCO','!6!e13!nCO','!6C!e18!nO','!6NH!d3!n','!6HCO!e+!n','!6CS','!6H!d2!nCO','HCN']
  speclabel = ['!6!e12!nCO (1-0)','!6!e12!nCO (2-1)','!6!e12!nCO (3-2)',$
               '!6!e13!nCO (1-0)','!6!e13!nCO (2-1)','!6!e13!nCO (3-2)','!6HI 21cm']
  struct_12c16o_10 = {MOL, fillab:'12C16O-10'}
  struct_12c16o_21 = {MOL, fillab:'12C16O-21'}
  struct_12c16o_32 = {MOL, fillab:'12C16O-32'}
  struct_13c16o_10 = {MOL, fillab:'13C16O-10'}
  struct_13c16o_21 = {MOL, fillab:'13C16O-21'}
  struct_13c16o_32 = {MOL, fillab:'13C16O-32'}
  struct_h21cm     = {MOL, fillab:'HI-21cm'}

  molcat    = replicate(struct_12c16o_10,nallmol+1)
  molcat[1] = struct_12c16o_21
  molcat[2] = struct_12c16o_32
  molcat[3] = struct_13c16o_10
  molcat[4] = struct_13c16o_21
  molcat[5] = struct_13c16o_32
  molcat[6] = struct_h21cm

  !PATH = !PATH + ":~/lib/astrolib"

  fitsheadrd = strarr(13)

  if (nmol lt 3) then chscl = 1.0 else chscl = 2.4
  xchsz    = 1.0*chscl
  ychsz    = 1.0*chscl
  chsz     = 1.0
  chth     = 3.0

;=====================================================================
; constants
  pi     = 4d0*atan(1d0)
  clight = 3.0d10
  kboltz = 1.381d-16
  hplanck= 6.626d-27
  
  nx     = 0l
  ny     = 0l
  nv     = 0l
  nbeam  = n_elements(beamwidth)

  mintbr = 0.0
  maxtbr = 0.0

  iseed  = 151l

  if (n_elements(angle) gt 0) then isangle=1 else isangle=0

  ilo      = icen-iwdt
  ihi      = icen+iwdt
  jlo      = jcen-iwdt
  jhi      = jcen+iwdt
  nar      = float(ihi-ilo+1)*float(jhi-jlo+1)
  
  spec1arr = fltarr(2,nbins,nmol,models)

  minh1    = fltarr(nmol)
  maxh1    = fltarr(nmol)
  minh1[*] = 1e30
  maxh1[*] = -1e30
  minh2    = fltarr(nmol,models)
  maxh2    = fltarr(nmol,models)

  varr0    = findgen(nbins)
  varr     = fltarr(nbins)

  for m=0,models-1 do begin
    numstr = string(dmp[m],format='(i4.4)')
    if (isangle eq 1) then astr = '_a'+string(angle[m],format='(i3.3)')+'_' else astr = ''
    for im=0,nmol-1 do begin
      tmol = molcat[whichmol[im]]
      fnm = disk+mdl+'/mollosvd'+astr+string(idir,format='(i1)')+'_'$
                +tmol.fillab+'_'+numstr+'c'+string(nbins,format='(i3.3)')+'s'$
                +string(beamwidth,format='(i2.2)')+'e'+string(fix(noise*100.0),format='(i3.3)')+'.fits'
      print,'MOLLOSVD_SPEC: reading file ',fnm
      fitsheadrd = strarr(13)
      sredcube = ptr_new(mrdfits(fnm,0,fitsheadrd))
      print,"MOLLOSVD_SPEC: fits header:"
      print,fitsheadrd
      reads,fitsheadrd[3],nx,    format='(11x,i20)'
      reads,fitsheadrd[4],ny,    format='(11x,i20)'
      reads,fitsheadrd[5],nv,    format='(11x,i20)'
      reads,fitsheadrd[6],minvph,format='(11x,e20.5)'
      reads,fitsheadrd[7],maxvph,format='(11x,e20.5)'
      reads,fitsheadrd[8],mintbr,format='(11x,e20.5)'
      reads,fitsheadrd[9],maxtbr,format='(11x,e20.5)'
      print,format='("MOLLOSVD_SPEC: MOLECU = ",a11)',tmol.fillab
      print,format='("MOLLOSVD_SPEC: NAXIS1 = ",i5)',nx
      print,format='("MOLLOSVD_SPEC: NAXIS2 = ",i5)',ny
      print,format='("MOLLOSVD_SPEC: NAXIS3 = ",i5)',nv
      print,format='("MOLLOSVD_SPEC: MINVLS = ",e13.5)',minvph
      print,format='("MOLLOSVD_SPEC: MAXVLS = ",e13.5)',maxvph
      print,format='("MOLLOSVD_SPEC: MINTBR = ",e13.5)',mintbr
      print,format='("MOLLOSVD_SPEC: MAXTBR = ",e13.5)',maxtbr


      dv = (maxvph-minvph)/float(nv)
      varr[*] = (0.5+varr0[*])*dv+minvph

      knum = 0l
      indx = ptr_new(where((*sredcube) lt 0.0,knum))
      if (knum gt 0) then begin
        (*sredcube)[*indx] = 0.0
        ptr_free,indx
      endif

;     centroid velocity and LOSVD
      svcenred = ptr_new(fltarr(nx,ny))
      svrmsred = ptr_new(fltarr(nx,ny))
      for iy=0,ny-1 do begin
        for ix=0,nx-1 do begin
          (*svcenred)[ix,iy] = total(varr[*]*(*sredcube)[ix,iy,*])/total(reform((*sredcube)[ix,iy,*]))
          (*svrmsred)[ix,iy] = sqrt( total((varr[*]-(*svcenred)[ix,iy])^2*reform((*sredcube)[ix,iy,*])) $
                                    /total(reform((*sredcube)[ix,iy,*])))
        endfor 
      endfor
      minsvcen = min((*svcenred))
      maxsvcen = max((*svcenred))
      minsvrms = min((*svrmsred))
      maxsvrms = max((*svrmsred))
      print,format='("MOLLOSVD_SPEC: min/max(svcen) = ",e13.5,e13.5)',minsvcen,maxsvcen
      print,format='("MOLLOSVD_SPEC: min/max(svrms) = ",e13.5,e13.5)',minsvrms,maxsvrms

;     1d spectrum through specified coordinate
      for iy=jlo,jhi do begin
        for ix=ilo,ihi do begin
          spec1arr[1,*,im,m] = spec1arr[1,*,im,m] + reform((*sredcube)[ix,iy,*])
        endfor
      endfor
      spec1arr[1,*,im,m] = spec1arr[1,*,im,m]/nar
      minh1[im] = min([minh1[im],min(spec1arr[1,*,im,m])])
      maxh1[im] = max([maxh1[im],max(spec1arr[1,*,im,m])])
      spec1arr[0,*,im,m] = varr[*]

;     spectrum map for overlay
      nxbox = nx/subsize
      nybox = ny/subsize
      print,"MOLLOSVD_SPEC: nxbox, nybox = ",nxbox,nybox
      spec2arr = ptr_new(fltarr(nbins,nmol,models,nxbox,nybox))
      histbox  = ptr_new(fltarr(4+2*nbins*nxbox*nybox))
      for jbox = 0,nybox-1 do begin
        for ibox = 0,nxbox-1 do begin
          for iy=jbox*subsize,(jbox+1)*subsize-1 do begin    
            for ix=ibox*subsize,(ibox+1)*subsize-1 do begin    
              (*spec2arr)[*,im,m,ibox,jbox] = (*spec2arr)[*,im,m,ibox,jbox] + reform((*sredcube)[ix,iy,*])/nar
            endfor
          endfor
        endfor
      endfor
      minh2[im,m] = min((*spec2arr)[*,im,m,*,*])      
      maxh2[im,m] = max((*spec2arr)[*,im,m,*,*])      

      (*histbox)[0] = float(nx)
      (*histbox)[1] = float(ny)
      (*histbox)[2] = float(subsize)
      (*histbox)[3] = float(nv)
      for l=0,nybox-1 do begin
        for k=0,nxbox-1 do begin
          for i=0,nv-1 do begin
            (*histbox)[i*2l*nxbox*nybox+1*nxbox*nybox+k*nybox+l+4l] = (*spec2arr)[i,im,m,k,l]
            (*histbox)[i*2l*nxbox*nybox+0*nxbox*nybox+k*nybox+l+4l] = varr[i]
          endfor
        endfor
      endfor

      surf       = ptr_new((total((*sredcube),3)))
;      pvplane    = ptr_new(transpose(reform((*sredcube)[nx/2,*,*])))
;      pvplane    = ptr_new(transpose(total((*sredcube),1)))
      pvplane    = ptr_new((total((*sredcube),2)/float(ny)))

      minmaxsurf = [minvph,maxvph]
      dminmax    = minmaxsurf

;     map of centroid velocity
      fnm = odisk+'/mollosvd'+astr+string(idir,format='(i1)')+'_'$
                +tmol.fillab+'_'+mdl+'_'+numstr+'c'+string(nv,format='(i3.3)')+'s'$
                +string(beamwidth,format='(i2.2)')+'e'+string(fix(noise*100.0),format='(i3.3)')+'vcen'
      print,'MOLLOSVD_SPEC: writing file ',fnm
      overvector,(*surf),(*surf),(*surf),(*svcenred),nx,ny,1.0,1.0,color=colorvcen,cont=contv,$
                 invert=0,print='ps',threshdata=threshdata,ifnm=fnm,dminmax=dminmax,$
                 minmaxsurf=minmaxsurf,$
                 legend=speclabel[whichmol[im]]+'!6 t = '+string(dtdmp*float(dmp[m])*convtime,format='(f4.1)')+' Myr',$
                 clegend=[1.5,255.0]

      minmaxsurf = [minsvrms,maxsvrms]
      dminmax    = minmaxsurf

;     map of LOSVD
      fnm = odisk+'/mollosvd'+astr+string(idir,format='(i1)')+'_'$
                +tmol.fillab+'_'+mdl+'_'+numstr+'c'+string(nv,format='(i3.3)')+'s'$
                +string(beamwidth,format='(i2.2)')+'e'+string(fix(noise*100.0),format='(i3.3)')+'vrms'
      print,'MOLLOSVD_SPEC: writing file ',fnm
      overvector,(*surf),(*surf),(*surf),(*svrmsred),nx,ny,1.0,1.0,color=colorvrms,cont=contv,$
                 invert=0,print='ps',threshdata=threshdata,ifnm=fnm,dminmax=dminmax,$
                 minmaxsurf=minmaxsurf,$
                 legend=speclabel[whichmol[im]]+'!6 t = '+string(dtdmp*float(dmp[m])*convtime,format='(f4.1)')+' Myr',$
                 clegend=[1.5,255.0]

      minmaxsurf = [0.0,max(*surf)]
      dminmax    = minmaxsurf

;     "histbox" spectra over grey scale map of cloud
      fnm = odisk+'/mollosvd'+astr+string(idir,format='(i1)')+'_'$
                +tmol.fillab+'_'+mdl+'_'+numstr+'c'+string(nv,format='(i3.3)')+'s'$
                +string(beamwidth,format='(i2.2)')+'e'+string(fix(noise*100.0),format='(i3.3)')
      print,'MOLLOSVD_SPEC: writing file ',fnm
      overvector,(*surf),(*surf),(*surf),(*surf),nx,ny,1.0,1.0,color=color,cont=cont,$
                 invert=invert,print='ps',threshdata=threshdata,ifnm=fnm,dminmax=dminmax,$
                 minmaxsurf=minmaxsurf,histbox=(*histbox),$
                 legend=speclabel[whichmol[im]]+'!6 t = '+string(dtdmp*float(dmp[m])*convtime,format='(f4.1)')+' Myr',$
                 clegend=[1.5,255.0]

      minmaxsurf = [0.0,max(*pvplane)]
      dminmax    = minmaxsurf

;     "pv"-plot
      fnm = odisk+'/mollosvd'+astr+string(idir,format='(i1)')+'_'$
                +tmol.fillab+'_'+mdl+'_'+numstr+'c'+string(nv,format='(i3.3)')+'s'$
                +string(beamwidth,format='(i2.2)')+'e'+string(fix(noise*100.0),format='(i3.3)')+'pv'
      print,'MOLLOSVD_SPEC: writing file ',fnm
      overvector,(*pvplane),(*pvplane),(*pvplane),(*pvplane),nx,nv,1.0,1.0,color=color,cont=cont,$
                 invert=invert,print='ps',threshdata=threshdata,ifnm=fnm,$
                 dminmax=dminmax,minmaxsurf=minmaxsurf,$
                 legend=speclabel[whichmol[im]]+'!6 t = '+string(dtdmp*float(dmp[m])*convtime,format='(f4.1)')+' Myr',$
                 clegend=[1.5,255.0]
     

      ptr_free,sredcube,surf,pvplane,svrmsred,svcenred,spec2arr,histbox

    endfor
  endfor
 

; center line spectrum
  minv = minvph
  maxv = maxvph
  minh = minh1
  maxh = maxh1
  dv   = maxv-minv
  minv = minv-0.05*dv
  maxv = maxv+0.05*dv
  dh   = maxh-minh
  minh = minh-0.05*dh
  maxh = maxh+0.05*dh
  dirstr = string(idir,format='(i1)')
  set_plot,'ps'
  fnm  = odisk+'mollosvd'+dirstr+'_'+descr+'_vlos1.eps'
  print,'MOLLOSVD_SPEC: writing file ',fnm
  device,filename=fnm,/encaps,/color,bits=8,xsize=15.0,ysize=5.0*nmol
  !p.multi=[0,1,nmol] 
  loadct,ctable
  dypos= 0.9/float(nmol)
  for im = 0,nmol-1 do begin
    if (im eq 0) then txchsz = xchsz else txchsz = 0.0001
    plot,[minv,maxv],[minh[im],maxh[im]],xtitle='!6v!dlos!n [km s!e-1!n]',ytitle='!6T [K km s!e-1!n]',$
         xcharsize=txchsz,ycharsize=ychsz,$
         xthick=3,ythick=3,charthick=3,/nodata,position=[0.15,0.10+im*dypos,1.00,0.10+(im+1)*dypos],xstyle=1,ystyle=1
    for m=0,models-1 do begin
      oplot,spec1arr[0,*,im,m],spec1arr[1,*,im,m],thick=4,psym=10,color=ctype[m]
      legend = "t = "+string(dtdmp*float(dmp[m])*convtime,format='(f4.1)')+' Myr'
      oplot,[minv+dv*0.05,minv+0.15*dv],[maxh[im]-(0.1+0.1*(m+1))*dh[im],maxh[im]-(0.1+0.1*(m+1))*dh[im]],$
            thick=4,color=ctype[m],line=0
      xyouts,minv+0.18*dv,maxh[im]-(0.12+0.1*(m+1))*dh[im],legend,charthick=3,charsize=chsz
;      if (finite(fwhm[im,m]) eq 1) then $
;        xyouts,minv+0.88*dv,maxh[im]-(0.12+0.1*(m+1))*dh[im],'!6'+string(fwhm[im,m],$
;               format='(f4.2)')+' km s!e-1!n',charthick=3,charsize=chsz
    endfor
    xyouts,minv+0.8*dv,maxh[im]-0.2*dh[im],speclabel[whichmol[im]],charsize=1.5,charthick=3
  endfor
  xyouts,minv+0.05*dv,maxh[nmol-1]-0.1*dh[nmol-1],legend0,charsize=chsz,charthick=3
  !p.multi=0
  device,/close
  set_plot,'x'
 
;===============================================================

  return
end

